/********************************************************************
*
* Created by zelon(Kim, Jinwook Korea)
* 
*   2005. 5. 7
*	ZCacheImage.cpp
*
*                                       http://www.wimy.com
*********************************************************************/

#include "stdafx.h"
#include "ZCacheImage.h"

using namespace std;

ZCacheImage & ZCacheImage::GetInstance()
{
	static ZCacheImage ins;
	return ins;
}

ZCacheImage::ZCacheImage()
:	m_bGoOn(true)
,	m_iCacheLeftIndex(-1)
,	m_iCacheRightIndex(-1)
,	m_bUseCache(true)
,	m_iLogCacheHit(0)
,	m_iLogCacheMiss(0)
,	m_lCacheSize(0)
,	m_iMaxCacheImageNum(10)
,	m_iMaximumCacheMemoryMB(50)
{
	m_hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);

	m_imageMap.clear();
	m_imageMapRev.clear();
	m_cacheData.clear();

	InitializeCriticalSection(&m_lock);
}

ZCacheImage::~ZCacheImage()
{
	m_bGoOn = false;
	SetEvent(m_hEvent);

	WaitForSingleObject(m_hThread, INFINITE);

	DebugPrintf("Cached Thread ended.");

	CloseHandle(m_hThread);
	CloseHandle(m_hEvent);

	DeleteCriticalSection(&m_lock);
}

void ZCacheImage::SetImageVector(std::vector < std::string > & v)
{
	ZCacheLock lock;

	m_imageVector = v;
	m_imageMap.clear();
	m_imageMapRev.clear();
	
	m_cacheData.clear();
	m_lCacheSize = 0;

	for ( size_t i = 0; i < m_imageVector.size(); ++i)
	{
		m_imageMap[(int)i] = m_imageVector[i];
		m_imageMapRev[m_imageVector[i]] = (int)i;
	}

	DebugPrintf("imageMapSize : %d", m_imageMap.size());
	DebugPrintf("imageVec : %d", m_imageVector.size());
}

void ZCacheImage::StartThread()
{
	DWORD dwThreadID;
	m_hThread = CreateThread(0, 0, ThreadFuncProxy, this, 0, &dwThreadID);

	// Cache  ϴ  
	if ( SetThreadPriority(m_hThread, THREAD_PRIORITY_BELOW_NORMAL) == FALSE )
	{
		_ASSERTE(!"Can't SetThreadPriority!");
	}
}

DWORD ZCacheImage::ThreadFuncProxy(LPVOID p)
{
	ZCacheImage * pThis = (ZCacheImage*)p;

	pThis->ThreadFunc();

	return 0;
}

/// ĳõǾ ִ ͵   εκ  ִָ ε ´.
int ZCacheImage::_GetFarthestIndexFromCurrentIndex()
{
	ZCacheLock lock;

	CacheMapIterator it, endIt = m_cacheData.end();

	int iFarthestIndex = m_iCurrentIndex;
	int iDistanceMax = 0;
	int iDistance = 0;
	int iTempIndex = 0;

	std::string strFarthest;

	for ( it = m_cacheData.begin(); it != endIt; ++it)
	{
		iTempIndex = m_imageMapRev[it->first];
		iDistance = abs(iTempIndex - m_iCurrentIndex);

		if ( iDistance > iDistanceMax )
		{
			iDistanceMax = iDistance;
			iFarthestIndex = iTempIndex;
			strFarthest = it->first;
		}
	}

	_ASSERTE(iFarthestIndex >= 0 );

	return iFarthestIndex;
}

bool ZCacheImage::CacheIndex(int iIndex)
{
	// ִ ĳ ũ⸦ Ѿ  ̻ ĳ ʴ´.
	if ( (m_lCacheSize / 1024 / 1024) > m_iMaximumCacheMemoryMB )
	{
		return false;
	}

	if ( iIndex < 0 ) iIndex = 0;
	if ( iIndex >= (int)m_imageVector.size() ) iIndex = (int)m_imageVector.size() - 1;
	if ( iIndex == m_iCurrentIndex ) return true;

	// ̹ ĳõǾ ִ ã´.
	bool bFound = false;
	std::string strFileName;

	strFileName = m_imageMap[iIndex];
	{
		ZCacheLock lock;
		if ( m_cacheData.count(strFileName) > 0)
		{
			// found!
			bFound = true;
		}
	}

	if ( bFound == false )	// ĳõǾ   оδ.
	{
		ZImage temp;
		if ( temp.LoadFromFile(strFileName) )
		{
			/*
			//if ( m_bNewChange) return false;	// 纸 ִ  ε ٲ   for  Ѵ.
			
			AddCacheData(strFileName, temp);

			if ( m_bNewChange) return false;	// 纸 ִ  ε ٲ   for  Ѵ.
			*/

			//  ̹   
			if ( (m_lCacheSize + temp.GetImageSize()) / 1024 / 1024 > m_iMaximumCacheMemoryMB )
			{
				int iTemp = 100;
				int iFarthestIndex = -1;

				do
				{
					// ĳõǾ ִ ͵    index  հ ã´.
					iFarthestIndex = _GetFarthestIndexFromCurrentIndex();

					if ( abs(iFarthestIndex - m_iCurrentIndex) <= abs(iIndex - m_iCurrentIndex ) )
					{
						return false;
					}
					else
					{
						//          Ŭϰ,   ĳ    Ȳ̴.

						//    clear Ѵ.
						{
							ZCacheLock lock;

							std::string strFindFileName = m_imageMap[iFarthestIndex];
							CacheMapIterator it = m_cacheData.find(m_imageMap[iFarthestIndex]);

							if ( it != m_cacheData.end() )
							{
								m_lCacheSize -= it->second.GetImageSize();
								m_cacheData.erase(it);

								DebugPrintf("Farthest one clear");
							}
							else
							{
								_ASSERTE(!"Can't find the cache data.");
								return false;
							}
						}

						//    뷮 Ȯ ٽ  ̹   ִ  ĳø üũѴ.
						if ( (m_lCacheSize + temp.GetImageSize()) / 1024 / 1024 <= m_iMaximumCacheMemoryMB )
						{
							AddCacheData(strFileName, temp);
						}

					}

					//  ѷ ϱ  10 .
					--iTemp;
				} while( iTemp > 0 );

				_ASSERTE(iTemp > 0 );
			}
			else
			{
				AddCacheData(strFileName, temp);
				if ( m_bNewChange) return false;	// 纸 ִ  ε ٲ   for  Ѵ.
			}
		}
	}
	return true;
}

void ZCacheImage::ThreadFunc()
{
	m_bNewChange = false;
	CacheMapIterator it, endit;
	int iPos = 0;
	int i = 0;

	while ( m_bGoOn )
	{
		iPos = 1;
		_ASSERTE((int)m_imageVector.size() == (int)m_imageMap.size());
		_ASSERTE(m_iCurrentIndex <= (int)m_imageMap.size());
		_ASSERTE(m_iCurrentIndex <= (int)m_imageVector.size());

#ifdef _DEBUG
		if ( m_cacheData.empty() )
		{
			_ASSERTE(m_lCacheSize == 0);
		}
#endif

		/// ĳ ͸   ĳø ش.
		//CheckCacheDataAndClear();

		//for ( i=0; i<m_iMaxCacheImageNum/2; ++i)
		for ( int i=0; i<100; ++i)
		{
			if ( m_bNewChange) break;	// 纸 ִ  ε ٲ   for  Ѵ.
			
			// left side
			if ( !CacheIndex(m_iCurrentIndex - iPos) ) break;

			// right side
			if ( !CacheIndex(m_iCurrentIndex + iPos) ) break;

			++iPos;
		}

		DebugPrintf("wait event");

		WaitForSingleObject(m_hEvent, INFINITE);
		m_bNewChange = false;

		DebugPrintf("Recv event\r\n");
	}
}

bool ZCacheImage::hasCachedData(const std::string & strFilename, int iIndex)
{
	if ( ! m_bUseCache ) return false;

	// index  üũѴ.
	m_iCurrentIndex = iIndex;
	m_bNewChange = true;

	SetEvent(m_hEvent);

	{
		ZCacheLock lock;
		
		if ( m_cacheData.count(strFilename) > 0 ) return true;
	}
	return false;
}

void ZCacheImage::getCachedData(const std::string & strFilename, ZImage & image)
{
	CacheMapIterator it;
	ZCacheLock lock;
	
	it = m_cacheData.find(strFilename);
	if ( it == m_cacheData.end() )
	{
		_ASSERTE(!"Can't get NOT CACHED");
	}
	else
	{
		DebugPrintf("Cache hit");
	}
	image = it->second;
}


void ZCacheImage::AddCacheData(const std::string & strFilename, ZImage & image)
{
	ZCacheLock lock;

	/// ̹ ĳõǾ  ĳ ʴ´.
	if ( m_cacheData.count(strFilename) > 0) return;

	/// 뷮 üũؼ  ̹ ĳ   Ѿ ĳ ʴ´.
	if ( (m_lCacheSize + image.GetImageSize()) /1024/1024 > m_iMaximumCacheMemoryMB ) return;

	DebugPrintf("%s added to cache", strFilename.c_str());

#ifdef _DEBUG
	m_cacheData[strFilename] = image;
	m_lCacheSize += m_cacheData[strFilename].GetImageSize();
#else
	try
	{
		m_cacheData[strFilename] = image;
	}
	catch ( ... )
	{
		return;
	}
	m_lCacheSize += m_cacheData[strFilename].GetImageSize();
#endif
}

long ZCacheImage::GetCachedKByte() const
{
	return (m_lCacheSize/1024);
}